(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('styled-components'), require('uniqid')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'styled-components', 'uniqid'], factory) :
  (global = global || self, factory(global['@vojtechportes/react-query-builder'] = {}, global.React, global.styled, global.uniqid));
}(this, function (exports, React, styled, uniqid) { 'use strict';

  var React__default = 'default' in React ? React['default'] : React;
  styled = styled && styled.hasOwnProperty('default') ? styled['default'] : styled;
  uniqid = uniqid && uniqid.hasOwnProperty('default') ? uniqid['default'] : uniqid;

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  function _taggedTemplateLiteralLoose(strings, raw) {
    if (!raw) {
      raw = strings.slice(0);
    }

    strings.raw = raw;
    return strings;
  }

  var clone = function clone(data) {
    return JSON.parse(JSON.stringify(data));
  };

  var isBoolean = function isBoolean(value) {
    return typeof value === 'boolean';
  };
  var isString = function isString(value) {
    return typeof value === 'string';
  };
  var isNumber = function isNumber(value) {
    return typeof value === 'number';
  };
  var isUndefined = function isUndefined(value) {
    return typeof value === 'undefined';
  };
  var isReactText = function isReactText(value) {
    return isString(value) || isNumber(value);
  };
  var isReactTextArray = function isReactTextArray(value) {
    return typeof value === 'object';
  };
  var isOptionList = function isOptionList(value) {
    return typeof value === 'object';
  };

  var FieldSelect = function FieldSelect(_ref) {
    var selectedValue = _ref.selectedValue,
        id = _ref.id;

    var _useContext = React.useContext(BuilderContext),
        fields = _useContext.fields,
        data = _useContext.data,
        setData = _useContext.setData,
        onChange = _useContext.onChange,
        components = _useContext.components;

    var Select = components.form.Select;

    var handleChange = function handleChange(value) {
      var clonedData = clone(data);
      var parentIndex = clonedData.findIndex(function (item) {
        return item.id === id;
      });
      var nextField = fields.filter(function (item) {
        return item.field === value;
      })[0];
      clonedData[parentIndex].field = value;
      delete clonedData[parentIndex].value;
      delete clonedData[parentIndex].operators;
      delete clonedData[parentIndex].operator;

      switch (nextField.type) {
        case 'BOOLEAN':
          clonedData[parentIndex].value = false;
          break;

        case 'DATE':
          if (nextField.operators && ['BETWEEN', 'NOT_BETWEEN'].includes(nextField.operators[0])) {
            clonedData[parentIndex].value = ['', ''];
          } else {
            clonedData[parentIndex].value = '';
          }

          clonedData[parentIndex].operator = nextField.operators && nextField.operators[0];
          clonedData[parentIndex].operators = nextField.operators;
          break;

        case 'TEXT':
          if (nextField.operators && ['BETWEEN', 'NOT_BETWEEN'].includes(nextField.operators[0])) {
            clonedData[parentIndex].value = ['', ''];
          } else {
            clonedData[parentIndex].value = '';
          }

          clonedData[parentIndex].operator = nextField.operators && nextField.operators[0];
          clonedData[parentIndex].operators = nextField.operators;
          break;

        case 'NUMBER':
          if (nextField.operators && ['BETWEEN', 'NOT_BETWEEN'].includes(nextField.operators[0])) {
            clonedData[parentIndex].value = [0, 0];
          } else {
            clonedData[parentIndex].value = 0;
          }

          clonedData[parentIndex].operator = nextField.operators && nextField.operators[0];
          clonedData[parentIndex].operators = nextField.operators;
          break;

        case 'LIST':
          if (isOptionList(nextField.value)) {
            clonedData[parentIndex].value = nextField.value[0].value;
          }

          clonedData[parentIndex].operator = nextField.operators && nextField.operators[0];
          clonedData[parentIndex].operators = nextField.operators;
          break;

        case 'MULTI_LIST':
          if (isOptionList(nextField.value)) {
            clonedData[parentIndex].value = [];
          }

          clonedData[parentIndex].operator = nextField.operators && nextField.operators[0];
          clonedData[parentIndex].operators = nextField.operators;
          break;

        case 'STATEMENT':
          clonedData[parentIndex].value = nextField.value;
          break;
      }

      setData(clonedData);
      onChange(clonedData);
    };

    var fieldNames = fields.map(function (field) {
      return {
        value: field.field,
        label: field.label
      };
    });
    return React__default.createElement(Select, {
      values: fieldNames,
      selectedValue: selectedValue,
      onChange: handleChange
    });
  };

  var Boolean = function Boolean(_ref) {
    var selectedValue = _ref.selectedValue,
        id = _ref.id;

    var _useContext = React.useContext(BuilderContext),
        data = _useContext.data,
        setData = _useContext.setData,
        onChange = _useContext.onChange,
        components = _useContext.components;

    var Switch = components.form.Switch;

    var handleChange = function handleChange(value) {
      var clonedData = clone(data);
      var parentIndex = clonedData.findIndex(function (item) {
        return item.id === id;
      });
      clonedData[parentIndex].value = value;
      setData(clonedData);
      onChange(clonedData);
    };

    return React__default.createElement(Switch, {
      onChange: handleChange,
      switched: selectedValue
    });
  };

  var Select = function Select(_ref) {
    var selectedValue = _ref.selectedValue,
        values = _ref.values,
        id = _ref.id;

    var _useContext = React.useContext(BuilderContext),
        data = _useContext.data,
        setData = _useContext.setData,
        onChange = _useContext.onChange,
        components = _useContext.components;

    var SelectBase = components.form.Select;

    var handleChange = function handleChange(value) {
      var clonedData = clone(data);
      var parentIndex = clonedData.findIndex(function (item) {
        return item.id === id;
      });
      clonedData[parentIndex].value = value;
      setData(clonedData);
      onChange(clonedData);
    };

    return React__default.createElement(SelectBase, {
      onChange: handleChange,
      selectedValue: selectedValue,
      values: values
    });
  };

  var SelectMulti = function SelectMulti(_ref) {
    var values = _ref.values,
        selectedValue = _ref.selectedValue,
        id = _ref.id;

    var _useContext = React.useContext(BuilderContext),
        data = _useContext.data,
        setData = _useContext.setData,
        onChange = _useContext.onChange,
        components = _useContext.components;

    var SelectMultiBase = components.form.SelectMulti;

    var handleChange = function handleChange(value) {
      if (setData && onChange) {
        var clonedData = clone(data);
        var parentIndex = clonedData.findIndex(function (item) {
          return item.id === id;
        });
        clonedData[parentIndex].value = clonedData[parentIndex].value.filter(function (item) {
          return item !== value;
        });
        clonedData[parentIndex].value.push(value);
        setData(clonedData);
        onChange(clonedData);
      }
    };

    var handleDelete = function handleDelete(value) {
      var clonedData = clone(data);
      var parentIndex = clonedData.findIndex(function (item) {
        return item.id === id;
      });
      clonedData[parentIndex].value = clonedData[parentIndex].value.filter(function (item) {
        return item !== value;
      });
      setData(clonedData);
      onChange(clonedData);
    };

    return React__default.createElement(SelectMultiBase, {
      onChange: handleChange,
      onDelete: handleDelete,
      selectedValue: selectedValue,
      values: values
    });
  };

  var OperatorSelect = function OperatorSelect(_ref) {
    var values = _ref.values,
        selectedValue = _ref.selectedValue,
        id = _ref.id;

    var _useContext = React.useContext(BuilderContext),
        fields = _useContext.fields,
        data = _useContext.data,
        setData = _useContext.setData,
        onChange = _useContext.onChange,
        components = _useContext.components;

    var Select = components.form.Select;

    var handleChange = function handleChange(value) {
      var clonedData = clone(data);
      var parentIndex = clonedData.findIndex(function (item) {
        return item.id === id;
      });
      var fieldIndex = fields.findIndex(function (item) {
        return clonedData[parentIndex].field === item.field;
      });

      if (['DATE', 'TEXT', 'NUMBER'].includes(fields[fieldIndex].type)) {
        if (!['BETWEEN', 'NOT_BETWEEN'].includes(value) && isReactTextArray(clonedData[parentIndex].value)) {
          clonedData[parentIndex].value = '';
        } else if (['BETWEEN', 'NOT_BETWEEN'].includes(value) && !isReactTextArray(clonedData[parentIndex].value)) {
          clonedData[parentIndex].value = ['', ''];
        }
      }

      clonedData[parentIndex].operator = value;
      setData(clonedData);
      onChange(clonedData);
    };

    return React__default.createElement(Select, {
      values: values,
      selectedValue: selectedValue,
      onChange: handleChange
    });
  };

  var Input = function Input(_ref) {
    var type = _ref.type,
        value = _ref.value,
        id = _ref.id;

    var _useContext = React.useContext(BuilderContext),
        data = _useContext.data,
        setData = _useContext.setData,
        onChange = _useContext.onChange,
        components = _useContext.components;

    var InputBase = components.form.Input;

    var handleChange = function handleChange(value, index) {
      var clonedData = clone(data);
      var parentIndex = clonedData.findIndex(function (item) {
        return item.id === id;
      });

      if (!isUndefined(index)) {
        clonedData[parentIndex].value[index] = value;
      } else {
        clonedData[parentIndex].value = value;
      }

      setData(clonedData);
      onChange(clonedData);
    };

    if (isReactTextArray(value)) {
      return React__default.createElement(React__default.Fragment, null, React__default.createElement(InputBase, {
        type: type,
        value: value[0],
        onChange: function onChange(value) {
          return handleChange(value, 0);
        }
      }), React__default.createElement(InputBase, {
        type: type,
        value: value[1],
        onChange: function onChange(value) {
          return handleChange(value, 1);
        }
      }));
    } else {
      return React__default.createElement(InputBase, {
        type: type,
        value: value,
        onChange: handleChange
      });
    }
  };

  function _templateObject() {
    var data = _taggedTemplateLiteralLoose(["\n  align-self: center;\n"]);

    _templateObject = function _templateObject() {
      return data;
    };

    return data;
  }
  var BooleanContainer = styled.div(_templateObject());
  var Component = function Component(_ref) {
    var fieldRef = _ref.field,
        selectedValue = _ref.value,
        operator = _ref.operator,
        id = _ref.id;

    var _useContext = React.useContext(BuilderContext),
        fields = _useContext.fields,
        data = _useContext.data,
        setData = _useContext.setData,
        onChange = _useContext.onChange,
        components = _useContext.components;

    var ComponentContainer = components.Component,
        Remove = components.Remove;

    var handleDelete = function handleDelete() {
      var clonedData = clone(data);
      var index = clonedData.findIndex(function (item) {
        return item.id === id;
      });
      var parentIndex = clonedData.findIndex(function (item) {
        return item.id === clonedData[index].parent;
      });
      var parent = clonedData[parentIndex];
      parent.children = parent.children.filter(function (item) {
        return item !== id;
      });
      clonedData = clonedData.filter(function (item) {
        return item.id !== id;
      });
      setData(clonedData);
      onChange(clonedData);
    };

    if (fields) {
      if (fieldRef === '') {
        return React__default.createElement(ComponentContainer, {
          controls: React__default.createElement(Remove, {
            label: "Delete",
            onClick: handleDelete
          })
        }, React__default.createElement(FieldSelect, {
          selectedValue: '',
          id: id
        }));
      } else {
        try {
          var fieldIndex = fields.findIndex(function (item) {
            return item.field === fieldRef;
          });
          var _fields$fieldIndex = fields[fieldIndex],
              field = _fields$fieldIndex.field,
              operators = _fields$fieldIndex.operators,
              type = _fields$fieldIndex.type,
              fieldValue = _fields$fieldIndex.value;
          var operatorsOptionList = operators && operators.map(function (item) {
            return {
              value: item,
              label: item
            };
          });
          return React__default.createElement(ComponentContainer, {
            controls: React__default.createElement(Remove, {
              label: "Delete",
              onClick: handleDelete
            })
          }, React__default.createElement(FieldSelect, {
            selectedValue: field,
            id: id
          }), type === 'BOOLEAN' && isBoolean(selectedValue) && React__default.createElement(BooleanContainer, null, React__default.createElement(Boolean, {
            id: id,
            selectedValue: selectedValue
          })), type === 'LIST' && isReactText(selectedValue) && isOptionList(fieldValue) && isOptionList(operatorsOptionList) && React__default.createElement(React__default.Fragment, null, React__default.createElement(OperatorSelect, {
            id: id,
            values: operatorsOptionList,
            selectedValue: operator
          }), React__default.createElement(Select, {
            id: id,
            selectedValue: selectedValue,
            values: fieldValue
          })), type === 'MULTI_LIST' && isOptionList(fieldValue) && isOptionList(operatorsOptionList) && isReactTextArray(selectedValue) && React__default.createElement(React__default.Fragment, null, React__default.createElement(OperatorSelect, {
            id: id,
            values: operatorsOptionList,
            selectedValue: operator
          }), React__default.createElement(SelectMulti, {
            id: id,
            values: fieldValue,
            selectedValue: selectedValue
          })), type === 'TEXT' && isOptionList(operatorsOptionList) && (isReactText(selectedValue) || isReactTextArray(selectedValue)) && React__default.createElement(React__default.Fragment, null, React__default.createElement(OperatorSelect, {
            id: id,
            values: operatorsOptionList,
            selectedValue: operator
          }), React__default.createElement(Input, {
            type: "text",
            value: selectedValue,
            id: id
          })), type === 'NUMBER' && isOptionList(operatorsOptionList) && (isReactText(selectedValue) || isReactTextArray(selectedValue)) && React__default.createElement(React__default.Fragment, null, React__default.createElement(OperatorSelect, {
            id: id,
            values: operatorsOptionList,
            selectedValue: operator
          }), React__default.createElement(Input, {
            type: "number",
            value: selectedValue,
            id: id
          })), type === 'DATE' && isOptionList(operatorsOptionList) && (isReactText(selectedValue) || isReactTextArray(selectedValue)) && React__default.createElement(React__default.Fragment, null, React__default.createElement(OperatorSelect, {
            id: id,
            values: operatorsOptionList,
            selectedValue: operator
          }), React__default.createElement(Input, {
            type: "date",
            value: selectedValue,
            id: id
          })));
        } catch (e) {
          console.error("Field \"" + fieldRef + "\" not found in fields definition.");
        }
      }
    }

    return null;
  };

  var Group = function Group(_ref) {
    var value = _ref.value,
        isNegated = _ref.isNegated,
        children = _ref.children,
        id = _ref.id,
        isRoot = _ref.isRoot;

    var _useContext = React.useContext(BuilderContext),
        components = _useContext.components,
        data = _useContext.data,
        setData = _useContext.setData,
        onChange = _useContext.onChange;

    var Add = components.Add,
        GroupContainer = components.Group,
        Option = components.GroupHeaderOption,
        Remove = components.Remove;

    var findIndex = function findIndex() {
      var clonedData = clone(data);
      var parentIndex = clonedData.findIndex(function (item) {
        return item.id === id;
      });
      var insertAfter = parentIndex;

      if (data[parentIndex].children && data[parentIndex].children.length > 0) {
        var lastChildren = clonedData[parentIndex].children.slice(-1)[0];
        insertAfter = clonedData.findIndex(function (item) {
          return item.id === lastChildren;
        });
      }

      return {
        insertAfter: insertAfter,
        parentIndex: parentIndex,
        clonedData: clonedData
      };
    };

    var addItem = function addItem(payload) {
      var _findIndex = findIndex(),
          insertAfter = _findIndex.insertAfter,
          parentIndex = _findIndex.parentIndex,
          clonedData = _findIndex.clonedData;

      if (!clonedData[parentIndex].children) {
        clonedData[insertAfter].children = [];
      }

      clonedData[parentIndex].children.push(payload.id);
      clonedData.splice(Number(insertAfter) + 1, 0, payload);
      setData(clonedData);
      onChange(clonedData);
    };

    var handleAddGroup = function handleAddGroup() {
      var EmptyGroup = {
        type: 'GROUP',
        value: 'AND',
        isNegated: false,
        id: uniqid(),
        parent: id,
        children: []
      };
      addItem(EmptyGroup);
    };

    var handleAddRule = function handleAddRule() {
      var EmptyRule = {
        field: '',
        id: uniqid(),
        parent: id
      };
      addItem(EmptyRule);
    };

    var handleChangeGroupType = function handleChangeGroupType(nextValue) {
      var _findIndex2 = findIndex(),
          clonedData = _findIndex2.clonedData,
          parentIndex = _findIndex2.parentIndex;

      clonedData[parentIndex].value = nextValue;
      setData(clonedData);
      onChange(clonedData);
    };

    var handleToggleNegateGroup = function handleToggleNegateGroup(nextValue) {
      var _findIndex3 = findIndex(),
          clonedData = _findIndex3.clonedData,
          parentIndex = _findIndex3.parentIndex;

      clonedData[parentIndex].isNegated = nextValue;
      setData(clonedData);
      onChange(clonedData);
    };

    var handleDeleteGroup = function handleDeleteGroup() {
      var clonedData = clone(data).filter(function (item) {
        return item.id !== id;
      });
      clonedData = clonedData.map(function (item) {
        if (item.children && item.children.length > 0) {
          item.children = item.children.filter(function (childId) {
            return childId !== id;
          });
        }

        return item;
      });
      setData(clonedData);
      onChange(clonedData);
    };

    return React__default.createElement(GroupContainer, {
      controlsLeft: React__default.createElement(React__default.Fragment, null, React__default.createElement(Option, {
        isSelected: !!isNegated,
        value: !isNegated,
        onClick: handleToggleNegateGroup
      }, "NOT"), React__default.createElement(Option, {
        isSelected: value === 'AND',
        value: "AND",
        onClick: handleChangeGroupType
      }, "AND"), React__default.createElement(Option, {
        isSelected: value === 'OR',
        value: "OR",
        onClick: handleChangeGroupType
      }, "OR")),
      controlsRight: React__default.createElement(React__default.Fragment, null, React__default.createElement(Add, {
        onClick: handleAddRule,
        label: "Add Rule"
      }), React__default.createElement(Add, {
        onClick: handleAddGroup,
        label: "Add Group"
      }), !isRoot && React__default.createElement(Remove, {
        onClick: handleDeleteGroup,
        label: "Delete"
      }))
    }, children);
  };

  var Iterator = function Iterator(_ref) {
    var originalData = _ref.originalData,
        filteredData = _ref.filteredData,
        _ref$isRoot = _ref.isRoot,
        isRoot = _ref$isRoot === void 0 ? true : _ref$isRoot;
    return React__default.createElement(React__default.Fragment, null, filteredData.map(function (item) {
      if (typeof item.children !== 'undefined') {
        var items = [];
        item.children.forEach(function (id) {
          items.push(originalData.filter(function (fitem) {
            return id === fitem.id;
          })[0]);
        });

        switch (item.type) {
          case 'GROUP':
            {
              var id = item.id,
                  value = item.value,
                  isNegated = item.isNegated;
              return React__default.createElement(Group, {
                key: id,
                value: value,
                isNegated: isNegated,
                id: id,
                isRoot: isRoot
              }, React__default.createElement(Iterator, {
                originalData: originalData,
                filteredData: items,
                isRoot: false
              }));
            }

          default:
            {
              var field = item.field,
                  _value = item.value,
                  _id = item.id,
                  operator = item.operator;
              return React__default.createElement("div", {
                key: item.id
              }, React__default.createElement(Component, {
                key: _id,
                field: field,
                value: _value,
                operator: operator,
                id: _id
              }), React__default.createElement(Iterator, {
                originalData: originalData,
                filteredData: items,
                isRoot: false
              }));
            }
        }
      } else {
        var _field = item.field,
            _value2 = item.value,
            _id2 = item.id,
            _operator = item.operator;
        return React__default.createElement(Component, {
          key: _id2,
          field: _field,
          value: _value2,
          operator: _operator,
          id: _id2
        });
      }
    }));
  };

  var colors = {
    primary: '#20639B',
    secondary: '#3CAEA3',
    tertiary: '#911803',
    light: '#f4f4f4',
    medium: '#d9d9d9',
    darker: '#bfbfbf',
    dark: '#848484',
    enabled: '#3caea3',
    disabled: '#9fb2b0'
  };

  var normalizeTree = function normalizeTree(data) {
    var clonedData = {
      children: clone(data)
    };
    var normalizedData = [];

    var run = function run(d, parentId) {
      if (parentId === void 0) {
        parentId = 0;
      }

      if (typeof d.children !== 'undefined') {
        var children = [];
        d.children.map(function (item) {
          if (parentId !== 0) {
            item.parent = parentId;
          }

          var tmpItem = clone(item);
          delete tmpItem.children;
          normalizedData.push(tmpItem);
          children.push(tmpItem.id);
          run(item, item.id);
        });

        if (parentId !== 0) {
          for (var _i = 0, _normalizedData = normalizedData; _i < _normalizedData.length; _i++) {
            var item = _normalizedData[_i];

            if (item.id === parentId) {
              item.children = children;
            }
          }
        }
      }
    };

    run(clonedData);
    return normalizedData;
  };

  var denormalizeTree = function denormalizeTree(data) {
    var clonedData = clone(data);
    var denormalizedData = clonedData.filter(function (item) {
      return !item.parent;
    });

    var run = function run(d, originalData) {
      d.map(function (item) {
        if (typeof item.children !== 'undefined') {
          var tmpItem = clone(item);
          delete item.children;
          delete item.id;
          delete item.parent;
          delete item.operators;
          item.children = [];
          tmpItem.children.map(function (id) {
            var clonedChildrenData = clone(originalData.filter(function (oItem) {
              return oItem.id === id;
            })[0]);
            delete clonedChildrenData.id;
            delete clonedChildrenData.parent;
            delete clonedChildrenData.operators;
            item.children.push(clonedChildrenData);
          });
          run(item.children, originalData);
        }
      });
    };

    run(denormalizedData, clonedData);
    return denormalizedData;
  };

  var assignIds = function assignIds(data) {
    data = {
      children: clone(data)
    };

    var run = function run(d) {
      if (typeof d.children !== 'undefined') {
        d.children = d.children.map(function (item) {
          item.id = uniqid();
          return run(item);
        });
      }

      return d;
    };

    return run(data).children;
  };

  function _templateObject$1() {
    var data = _taggedTemplateLiteralLoose(["\n  color: #fff;\n  border: 0;\n  background-color: ", ";\n  padding: 0.5rem 1rem;\n  font-size: 0.7rem;\n  white-space: nowrap;\n  cursor: pointer;\n  border-radius: 3px;\n  outline: none;\n"]);

    _templateObject$1 = function _templateObject() {
      return data;
    };

    return data;
  }
  var StyledButton = styled.button(_templateObject$1(), colors.primary);
  var Button = function Button(_ref) {
    var onClick = _ref.onClick,
        className = _ref.className,
        label = _ref.label;
    return React__default.createElement(StyledButton, {
      onClick: onClick,
      className: className
    }, label);
  };

  function _templateObject$2() {
    var data = _taggedTemplateLiteralLoose(["\n  background-color: ", ";\n"]);

    _templateObject$2 = function _templateObject() {
      return data;
    };

    return data;
  }
  var SecondaryButton =
  /*#__PURE__*/
  styled(Button)(_templateObject$2(), colors.tertiary);

  function _templateObject$3() {
    var data = _taggedTemplateLiteralLoose(["\n  padding: 0.2rem 0.5rem;\n  height: 1.6rem;\n  line-height: 1.6rem;\n  border: 1px solid ", ";\n  border-radius: 3px;\n"]);

    _templateObject$3 = function _templateObject() {
      return data;
    };

    return data;
  }
  var StyledInput = styled.input(_templateObject$3(), colors.medium);
  var Input$1 = function Input(_ref) {
    var type = _ref.type,
        value = _ref.value,
        onChange = _ref.onChange,
        className = _ref.className;

    var handleChange = function handleChange(event) {
      onChange(event.target.value);
    };

    return React__default.createElement(StyledInput, {
      type: type,
      value: value,
      onChange: handleChange,
      className: className
    });
  };

  function _templateObject$4() {
    var data = _taggedTemplateLiteralLoose(["\n  padding: 0.2rem 0.5rem;\n  height: 1.6rem;\n  line-height: 1.6rem;\n  border: 1px solid ", ";\n  border-radius: 3px;\n"]);

    _templateObject$4 = function _templateObject() {
      return data;
    };

    return data;
  }
  var StyledSelect = styled.select(_templateObject$4(), colors.medium);
  var Select$1 = function Select(_ref) {
    var values = _ref.values,
        selectedValue = _ref.selectedValue,
        onChange = _ref.onChange,
        className = _ref.className;

    var handleChange = function handleChange(event) {
      onChange(event.target.value);
    };

    return React__default.createElement(StyledSelect, {
      onChange: handleChange,
      value: selectedValue,
      className: className
    }, React__default.createElement("option", {
      value: "",
      disabled: true
    }, "Select your option"), values.map(function (_ref2) {
      var optionValue = _ref2.value,
          optionLabel = _ref2.label;
      return React__default.createElement("option", {
        value: optionValue,
        key: optionValue
      }, optionLabel);
    }));
  };

  function _templateObject4() {
    var data = _taggedTemplateLiteralLoose(["\n  cursor: pointer;\n"]);

    _templateObject4 = function _templateObject4() {
      return data;
    };

    return data;
  }

  function _templateObject3() {
    var data = _taggedTemplateLiteralLoose(["\n  color: ", ";\n  border: 1px solid ", ";\n  border-radius: 3rem;\n  padding: 0.3rem 0.5rem;\n  font-size: 0.7rem;\n  line-height: 0.7rem;\n  white-space: nowrap;\n"]);

    _templateObject3 = function _templateObject3() {
      return data;
    };

    return data;
  }

  function _templateObject2() {
    var data = _taggedTemplateLiteralLoose(["\n  display: grid;\n  grid-auto-flow: column;\n  grid-auto-columns: min-content;\n  grid-auto--rows: min-content;\n  align-self: center;\n  grid-gap: 0.5rem;\n"]);

    _templateObject2 = function _templateObject2() {
      return data;
    };

    return data;
  }

  function _templateObject$5() {
    var data = _taggedTemplateLiteralLoose(["\n  display: grid;\n  grid-auto-columns: min-content;\n  grid-auto-flow: column;\n  grid-gap: 0.5rem;\n"]);

    _templateObject$5 = function _templateObject() {
      return data;
    };

    return data;
  }
  var Container = styled.div(_templateObject$5());
  var OptionContainer = styled.div(_templateObject2());
  var Option = styled.span(_templateObject3(), colors.dark, colors.dark);
  var Delete = styled.span(_templateObject4());
  var SelectMulti$1 = function SelectMulti(_ref) {
    var onChange = _ref.onChange,
        onDelete = _ref.onDelete,
        selectedValue = _ref.selectedValue,
        values = _ref.values;
    return React__default.createElement(Container, null, React__default.createElement(Select$1, {
      onChange: onChange,
      selectedValue: "",
      values: values
    }), React__default.createElement(OptionContainer, null, selectedValue.map(function (value) {
      var labelIndex = values.findIndex(function (item) {
        return item.value === value;
      });
      return React__default.createElement(Option, null, values[labelIndex].label, ' ', React__default.createElement(Delete, {
        onClick: function onClick() {
          return onDelete(value);
        }
      }, "[x]"));
    })));
  };

  function _templateObject2$1() {
    var data = _taggedTemplateLiteralLoose(["\n  width: 2.2rem;\n  height: 1rem;\n  position: relative;\n  border-radius: 1.4rem;\n  background-color: ", ";\n  transition: all 0.5s;\n  cursor: pointer;\n\n  ", " {\n    transition: all 0.5s;\n    left: ", ";\n  }\n"]);

    _templateObject2$1 = function _templateObject2() {
      return data;
    };

    return data;
  }

  function _templateObject$6() {
    var data = _taggedTemplateLiteralLoose(["\n  position: absolute;\n  width: 1rem;\n  height: 1rem;\n  background: white;\n  border-radius: 50%;\n  border: 1px solid ", ";\n"]);

    _templateObject$6 = function _templateObject() {
      return data;
    };

    return data;
  }
  var Knob = styled.div(_templateObject$6(), colors.medium);
  var StyledSwitch = styled.div(_templateObject2$1(), function (_ref) {
    var switched = _ref.switched;
    return switched ? colors.enabled : colors.disabled;
  }, Knob, function (_ref2) {
    var switched = _ref2.switched;
    return switched ? '1.2rem' : '0rem';
  });
  var Switch = function Switch(_ref3) {
    var switched = _ref3.switched,
        onChange = _ref3.onChange,
        className = _ref3.className;

    var handleClick = function handleClick() {
      if (onChange) {
        onChange(!switched);
      }
    };

    return React__default.createElement(StyledSwitch, {
      switched: switched,
      onClick: handleClick,
      className: className
    }, React__default.createElement(Knob, null));
  };

  function _templateObject3$1() {
    var data = _taggedTemplateLiteralLoose(["\n  display: grid;\n  grid-auto-flow: column;\n  grid-auto-columns: min-content;\n  grid-gap: 0.5rem;\n  justify-self: flex-end;\n"]);

    _templateObject3$1 = function _templateObject3() {
      return data;
    };

    return data;
  }

  function _templateObject2$2() {
    var data = _taggedTemplateLiteralLoose(["\n  display: grid;\n  grid-auto-columns: min-content;\n  grid-auto-flow: column;\n  grid-gap: 0.5rem;\n"]);

    _templateObject2$2 = function _templateObject2() {
      return data;
    };

    return data;
  }

  function _templateObject$7() {
    var data = _taggedTemplateLiteralLoose(["\n  display: grid;\n  grid-auto-flow: column;\n  padding: 0.7rem;\n  margin: 0.5rem 0;\n  border: 1px solid ", ";\n  background-color: #fff;\n"]);

    _templateObject$7 = function _templateObject() {
      return data;
    };

    return data;
  }
  var StyledComponent = styled.div(_templateObject$7(), colors.medium);
  var Content = styled.div(_templateObject2$2());
  var Controls = styled.div(_templateObject3$1());
  var Component$1 = function Component(_ref) {
    var children = _ref.children,
        controls = _ref.controls,
        className = _ref.className;
    return React__default.createElement(StyledComponent, {
      className: className
    }, React__default.createElement(Content, null, children), React__default.createElement(Controls, null, controls));
  };

  function _templateObject4$1() {
    var data = _taggedTemplateLiteralLoose(["\n  display: grid;\n  grid-auto-columns: min-content;\n  grid-auto-flow: column;\n  grid-gap: 0.5rem;\n  justify-self: flex-end;\n"]);

    _templateObject4$1 = function _templateObject4() {
      return data;
    };

    return data;
  }

  function _templateObject3$2() {
    var data = _taggedTemplateLiteralLoose(["\n  display: grid;\n  grid-auto-columns: min-content;\n  grid-auto-flow: column;\n  align-self: flex-end;\n  justify-self: flex-start;\n"]);

    _templateObject3$2 = function _templateObject3() {
      return data;
    };

    return data;
  }

  function _templateObject2$3() {
    var data = _taggedTemplateLiteralLoose(["\n  display: grid;\n  grid-template-columns: 1fr 1fr;\n  grid-gap: 1rem;\n  padding: 0 0 0.5rem;\n  border-bottom: 1px solid ", ";\n"]);

    _templateObject2$3 = function _templateObject2() {
      return data;
    };

    return data;
  }

  function _templateObject$8() {
    var data = _taggedTemplateLiteralLoose(["\n  background: #f4f4f4;\n  border: 1px solid ", ";\n  padding: 0.7rem;\n  margin: 0.5rem 0;\n  box-shadow: 0px 0px 5px -1px rgba(0, 0, 0, 0.15);\n"]);

    _templateObject$8 = function _templateObject() {
      return data;
    };

    return data;
  }
  var StyledGroup = styled.div(_templateObject$8(), colors.medium);
  var GroupHeader = styled.div(_templateObject2$3(), colors.medium);
  var Left = styled.div(_templateObject3$2());
  var Right = styled.div(_templateObject4$1());
  var Group$1 = function Group(_ref) {
    var controlsLeft = _ref.controlsLeft,
        controlsRight = _ref.controlsRight,
        children = _ref.children,
        className = _ref.className;
    return React__default.createElement(StyledGroup, {
      className: className
    }, React__default.createElement(GroupHeader, null, React__default.createElement(Left, null, controlsLeft), React__default.createElement(Right, null, controlsRight)), children);
  };

  function _templateObject$9() {
    var data = _taggedTemplateLiteralLoose(["\n  padding: 0.3rem 0.4rem;\n  font-weight: bold;\n  font-size: 0.7rem;\n  color: #fff;\n  background: ", ";\n  cursor: pointer;\n"]);

    _templateObject$9 = function _templateObject() {
      return data;
    };

    return data;
  }
  var StyledOption = styled.div(_templateObject$9(), function (_ref) {
    var isSelected = _ref.isSelected;
    return !!isSelected ? colors.enabled : colors.disabled;
  });
  var Option$1 = function Option(_ref2) {
    var children = _ref2.children,
        onClick = _ref2.onClick,
        value = _ref2.value,
        isSelected = _ref2.isSelected,
        className = _ref2.className;

    var handleClick = function handleClick() {
      onClick(value);
    };

    return React__default.createElement(StyledOption, {
      className: className,
      isSelected: isSelected,
      onClick: handleClick
    }, children);
  };

  function _templateObject$a() {
    var data = _taggedTemplateLiteralLoose(["\n  padding: 1rem;\n  border: 1px solid ", ";\n  background: #fff;\n"]);

    _templateObject$a = function _templateObject() {
      return data;
    };

    return data;
  }
  var StyledBuilder = styled.div(_templateObject$a(), colors.light);
  var BuilderContext =
  /*#__PURE__*/
  React.createContext({});
  var defaultComponents = {
    form: {
      Input: Input$1,
      Select: Select$1,
      SelectMulti: SelectMulti$1,
      Switch: Switch
    },
    Remove: SecondaryButton,
    Add: Button,
    Component: Component$1,
    Group: Group$1,
    GroupHeaderOption: Option$1
  };
  var Builder = function Builder(_ref) {
    var originalData = _ref.data,
        fields = _ref.fields,
        _ref$components = _ref.components,
        components = _ref$components === void 0 ? defaultComponents : _ref$components,
        onChange = _ref.onChange;
    var normalizedData;
    originalData = assignIds(originalData);
    components = _extends({}, defaultComponents, {}, components, {
      form: _extends({}, defaultComponents.form, {}, components.form)
    });

    if (originalData.length === 0) {
      originalData = [{
        type: 'GROUP',
        value: 'AND',
        isNegated: false,
        id: uniqid(),
        children: originalData
      }];
    }

    try {
      normalizedData = normalizeTree(originalData);
    } catch (e) {
      throw new Error('Input data tree is in invalid format');
    }

    var _useState = React.useState(normalizedData),
        data = _useState[0],
        setData = _useState[1];

    var filteredData = data.filter(function (item) {
      return !item.parent;
    });
    React.useEffect(function () {
      handleChange(originalData);
    }, []);

    var handleChange = function handleChange(nextData) {
      if (onChange) {
        try {
          onChange(denormalizeTree(nextData));
        } catch (e) {
          throw new Error('Input data tree is in invalid format');
        }
      }
    };

    return React__default.createElement(BuilderContext.Provider, {
      value: {
        fields: fields,
        components: components,
        data: data,
        setData: setData,
        onChange: handleChange
      }
    }, React__default.createElement(StyledBuilder, null, React__default.createElement(Iterator, {
      originalData: data,
      filteredData: filteredData
    })));
  };

  exports.Builder = Builder;
  exports.Button = Button;
  exports.Component = Component$1;
  exports.Group = Group$1;
  exports.GroupHeaderOption = Option$1;
  exports.Input = Input$1;
  exports.SecondaryButton = SecondaryButton;
  exports.Select = Select$1;
  exports.SelectMulti = SelectMulti$1;
  exports.Switch = Switch;
  exports.colors = colors;
  exports.defaultComponents = defaultComponents;

}));
//# sourceMappingURL=react-query-builder.umd.development.js.map
